Package: anota
Version: 1.60.0
Date: 2011-03-03
Title: ANalysis Of Translational Activity (ANOTA).
Author: Ola Larsson <ola.larsson@ki.se>, Nahum Sonenberg
        <nahum.sonenberg@mcgill.ca>, Robert Nadon
        <robert.nadon@mcgill.ca>
Maintainer: Ola Larsson <ola.larsson@ki.se>
Description: Genome wide studies of translational control is emerging
        as a tool to study verious biological conditions. The output
        from such analysis is both the mRNA level (e.g. cytosolic mRNA
        level) and the levl of mRNA actively involved in translation
        (the actively translating mRNA level) for each mRNA. The
        standard analysis of such data strives towards identifying
        differential translational between two or more sample classes -
        i.e. differences in actively translated mRNA levels that are
        independent of underlying differences in cytosolic mRNA levels.
        This package allows for such analysis using partial variances
        and the random variance model. As 10s of thousands of mRNAs are
        analyzed in parallell the library performs a number of tests to
        assure that the data set is suitable for such analysis.
Imports: multtest, qvalue
Depends: qvalue
LazyData: yes
LazyLoad: yes
License: GPL-3
biocViews: GeneExpression, DifferentialExpression, Microarray,
        Sequencing
git_url: https://git.bioconductor.org/packages/anota
git_branch: RELEASE_3_23
git_last_commit: dd09d4d
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 20:36:03 UTC; biocbuild
